# generated by datamodel-codegen:
#   filename:  bom-1.4.schema.json
#   timestamp: 2022-04-27T20:56:27+00:00

from __future__ import annotations

from enum import Enum


class Schema(Enum):
    AFL_2_0 = "AFL-2.0"
    AAL = "AAL"
    Adobe_2006 = "Adobe-2006"
    AFL_3_0 = "AFL-3.0"
    ADSL = "ADSL"
    field_0BSD = "0BSD"
    Afmparse = "Afmparse"
    AFL_1_2 = "AFL-1.2"
    AGPL_1_0_or_later = "AGPL-1.0-or-later"
    AFL_2_1 = "AFL-2.1"
    AFL_1_1 = "AFL-1.1"
    AGPL_1_0 = "AGPL-1.0"
    Adobe_Glyph = "Adobe-Glyph"
    AMDPLPA = "AMDPLPA"
    Aladdin = "Aladdin"
    ANTLR_PD = "ANTLR-PD"
    AML = "AML"
    Apache_1_0 = "Apache-1.0"
    ANTLR_PD_fallback = "ANTLR-PD-fallback"
    Abstyles = "Abstyles"
    AGPL_1_0_only = "AGPL-1.0-only"
    APAFML = "APAFML"
    APSL_1_0 = "APSL-1.0"
    APSL_1_1 = "APSL-1.1"
    APSL_2_0 = "APSL-2.0"
    AGPL_3_0_only = "AGPL-3.0-only"
    Apache_1_1 = "Apache-1.1"
    Apache_2_0 = "Apache-2.0"
    APL_1_0 = "APL-1.0"
    Bahyph = "Bahyph"
    Artistic_1_0 = "Artistic-1.0"
    AMPAS = "AMPAS"
    Barr = "Barr"
    AGPL_3_0_or_later = "AGPL-3.0-or-later"
    BlueOak_1_0_0 = "BlueOak-1.0.0"
    Beerware = "Beerware"
    Artistic_1_0_cl8 = "Artistic-1.0-cl8"
    blessing = "blessing"
    Borceux = "Borceux"
    BSD_2_Clause_NetBSD = "BSD-2-Clause-NetBSD"
    BSD_1_Clause = "BSD-1-Clause"
    BSD_2_Clause_Patent = "BSD-2-Clause-Patent"
    BitTorrent_1_0 = "BitTorrent-1.0"
    BSD_2_Clause_FreeBSD = "BSD-2-Clause-FreeBSD"
    BSD_3_Clause_Attribution = "BSD-3-Clause-Attribution"
    BSD_2_Clause = "BSD-2-Clause"
    APSL_1_2 = "APSL-1.2"
    BSD_3_Clause_LBNL = "BSD-3-Clause-LBNL"
    Artistic_2_0 = "Artistic-2.0"
    BSD_3_Clause_No_Nuclear_License_2014 = "BSD-3-Clause-No-Nuclear-License-2014"
    BSD_3_Clause_Modification = "BSD-3-Clause-Modification"
    BSD_4_Clause_Shortened = "BSD-4-Clause-Shortened"
    BSD_3_Clause = "BSD-3-Clause"
    BSD_3_Clause_Open_MPI = "BSD-3-Clause-Open-MPI"
    BitTorrent_1_1 = "BitTorrent-1.1"
    BSD_3_Clause_No_Nuclear_Warranty = "BSD-3-Clause-No-Nuclear-Warranty"
    BSD_Source_Code = "BSD-Source-Code"
    BSD_Protection = "BSD-Protection"
    AGPL_3_0 = "AGPL-3.0"
    BUSL_1_1 = "BUSL-1.1"
    Artistic_1_0_Perl = "Artistic-1.0-Perl"
    BSL_1_0 = "BSL-1.0"
    BSD_2_Clause_Views = "BSD-2-Clause-Views"
    CAL_1_0_Combined_Work_Exception = "CAL-1.0-Combined-Work-Exception"
    CATOSL_1_1 = "CATOSL-1.1"
    bzip2_1_0_5 = "bzip2-1.0.5"
    bzip2_1_0_6 = "bzip2-1.0.6"
    CC_BY_2_5 = "CC-BY-2.5"
    CC_BY_3_0_AT = "CC-BY-3.0-AT"
    C_UDA_1_0 = "C-UDA-1.0"
    CC_BY_3_0_US = "CC-BY-3.0-US"
    CC_BY_1_0 = "CC-BY-1.0"
    CC_BY_NC_1_0 = "CC-BY-NC-1.0"
    CC_BY_NC_2_0 = "CC-BY-NC-2.0"
    CC_BY_NC_2_5 = "CC-BY-NC-2.5"
    CC_BY_2_0 = "CC-BY-2.0"
    CC_BY_NC_4_0 = "CC-BY-NC-4.0"
    CC_BY_NC_ND_1_0 = "CC-BY-NC-ND-1.0"
    CC_BY_NC_ND_2_0 = "CC-BY-NC-ND-2.0"
    CC_BY_NC_3_0 = "CC-BY-NC-3.0"
    CC_BY_NC_ND_3_0_IGO = "CC-BY-NC-ND-3.0-IGO"
    CC_BY_NC_ND_3_0 = "CC-BY-NC-ND-3.0"
    BSD_3_Clause_No_Nuclear_License = "BSD-3-Clause-No-Nuclear-License"
    CC_BY_NC_ND_4_0 = "CC-BY-NC-ND-4.0"
    CC_BY_NC_SA_2_0 = "CC-BY-NC-SA-2.0"
    CC_BY_NC_SA_2_5 = "CC-BY-NC-SA-2.5"
    CC_BY_NC_SA_3_0 = "CC-BY-NC-SA-3.0"
    CC_BY_NC_SA_4_0 = "CC-BY-NC-SA-4.0"
    CC_BY_ND_1_0 = "CC-BY-ND-1.0"
    BSD_3_Clause_Clear = "BSD-3-Clause-Clear"
    CC_BY_ND_2_5 = "CC-BY-ND-2.5"
    CC_BY_ND_3_0 = "CC-BY-ND-3.0"
    CC_BY_ND_4_0 = "CC-BY-ND-4.0"
    CC_BY_SA_1_0 = "CC-BY-SA-1.0"
    CC_BY_SA_2_0_UK = "CC-BY-SA-2.0-UK"
    CC_BY_SA_2_0 = "CC-BY-SA-2.0"
    CC_BY_SA_2_1_JP = "CC-BY-SA-2.1-JP"
    CC_BY_ND_2_0 = "CC-BY-ND-2.0"
    CC_BY_SA_3_0_AT = "CC-BY-SA-3.0-AT"
    CC_BY_SA_3_0 = "CC-BY-SA-3.0"
    CC_BY_SA_4_0 = "CC-BY-SA-4.0"
    CC_BY_SA_2_5 = "CC-BY-SA-2.5"
    CC_BY_3_0 = "CC-BY-3.0"
    CDDL_1_0 = "CDDL-1.0"
    CC0_1_0 = "CC0-1.0"
    CC_BY_NC_ND_2_5 = "CC-BY-NC-ND-2.5"
    CC_BY_NC_SA_1_0 = "CC-BY-NC-SA-1.0"
    Caldera = "Caldera"
    CDLA_Permissive_1_0 = "CDLA-Permissive-1.0"
    CC_BY_4_0 = "CC-BY-4.0"
    CC_PDDC = "CC-PDDC"
    BSD_4_Clause_UC = "BSD-4-Clause-UC"
    BSD_4_Clause = "BSD-4-Clause"
    CAL_1_0 = "CAL-1.0"
    CDDL_1_1 = "CDDL-1.1"
    CERN_OHL_1_2 = "CERN-OHL-1.2"
    CERN_OHL_1_1 = "CERN-OHL-1.1"
    CERN_OHL_P_2_0 = "CERN-OHL-P-2.0"
    CERN_OHL_S_2_0 = "CERN-OHL-S-2.0"
    CECILL_1_1 = "CECILL-1.1"
    CECILL_2_0 = "CECILL-2.0"
    CECILL_1_0 = "CECILL-1.0"
    CNRI_Python = "CNRI-Python"
    CNRI_Python_GPL_Compatible = "CNRI-Python-GPL-Compatible"
    copyleft_next_0_3_0 = "copyleft-next-0.3.0"
    CPAL_1_0 = "CPAL-1.0"
    copyleft_next_0_3_1 = "copyleft-next-0.3.1"
    CPL_1_0 = "CPL-1.0"
    ClArtistic = "ClArtistic"
    CECILL_C = "CECILL-C"
    CNRI_Jython = "CNRI-Jython"
    Condor_1_1 = "Condor-1.1"
    CPOL_1_02 = "CPOL-1.02"
    curl = "curl"
    diffmark = "diffmark"
    Crossword = "Crossword"
    Dotseqn = "Dotseqn"
    DOC = "DOC"
    DSDP = "DSDP"
    DRL_1_0 = "DRL-1.0"
    ECL_1_0 = "ECL-1.0"
    ECL_2_0 = "ECL-2.0"
    eCos_2_0 = "eCos-2.0"
    CrystalStacker = "CrystalStacker"
    CERN_OHL_W_2_0 = "CERN-OHL-W-2.0"
    D_FSL_1_0 = "D-FSL-1.0"
    eGenix = "eGenix"
    EPICS = "EPICS"
    Entessa = "Entessa"
    EPL_1_0 = "EPL-1.0"
    EFL_2_0 = "EFL-2.0"
    CUA_OPL_1_0 = "CUA-OPL-1.0"
    etalab_2_0 = "etalab-2.0"
    EUPL_1_0 = "EUPL-1.0"
    ErlPL_1_1 = "ErlPL-1.1"
    EUDatagrid = "EUDatagrid"
    EUPL_1_1 = "EUPL-1.1"
    Cube = "Cube"
    dvipdfm = "dvipdfm"
    FreeBSD_DOC = "FreeBSD-DOC"
    Eurosym = "Eurosym"
    FSFAP = "FSFAP"
    FreeImage = "FreeImage"
    FSFULLR = "FSFULLR"
    FSFUL = "FSFUL"
    GD = "GD"
    GFDL_1_1_invariants_only = "GFDL-1.1-invariants-only"
    EUPL_1_2 = "EUPL-1.2"
    EPL_2_0 = "EPL-2.0"
    GFDL_1_1_no_invariants_or_later = "GFDL-1.1-no-invariants-or-later"
    GFDL_1_1_only = "GFDL-1.1-only"
    GFDL_1_1_or_later = "GFDL-1.1-or-later"
    GFDL_1_1 = "GFDL-1.1"
    FTL = "FTL"
    GFDL_1_2_invariants_or_later = "GFDL-1.2-invariants-or-later"
    GFDL_1_1_invariants_or_later = "GFDL-1.1-invariants-or-later"
    GFDL_1_2_invariants_only = "GFDL-1.2-invariants-only"
    GFDL_1_2_only = "GFDL-1.2-only"
    GFDL_1_2_or_later = "GFDL-1.2-or-later"
    GFDL_1_2 = "GFDL-1.2"
    GFDL_1_2_no_invariants_only = "GFDL-1.2-no-invariants-only"
    GFDL_1_3_invariants_only = "GFDL-1.3-invariants-only"
    GFDL_1_3_no_invariants_only = "GFDL-1.3-no-invariants-only"
    GFDL_1_3_invariants_or_later = "GFDL-1.3-invariants-or-later"
    GFDL_1_2_no_invariants_or_later = "GFDL-1.2-no-invariants-or-later"
    Fair = "Fair"
    Frameworx_1_0 = "Frameworx-1.0"
    Giftware = "Giftware"
    GFDL_1_1_no_invariants_only = "GFDL-1.1-no-invariants-only"
    GL2PS = "GL2PS"
    Glulxe = "Glulxe"
    Glide = "Glide"
    gnuplot = "gnuplot"
    GLWTPL = "GLWTPL"
    GPL_1_0_only = "GPL-1.0-only"
    GPL_1_0_or_later = "GPL-1.0-or-later"
    GPL_1_0 = "GPL-1.0"
    GFDL_1_3_only = "GFDL-1.3-only"
    GPL_2_0_only = "GPL-2.0-only"
    GPL_2_0_or_later = "GPL-2.0-or-later"
    GPL_2_0_with_autoconf_exception = "GPL-2.0-with-autoconf-exception"
    GPL_2_0_ = "GPL-2.0+"
    GFDL_1_3 = "GFDL-1.3"
    GPL_1_0_ = "GPL-1.0+"
    CDLA_Sharing_1_0 = "CDLA-Sharing-1.0"
    GPL_2_0_with_classpath_exception = "GPL-2.0-with-classpath-exception"
    GPL_2_0_with_GCC_exception = "GPL-2.0-with-GCC-exception"
    GPL_2_0_with_bison_exception = "GPL-2.0-with-bison-exception"
    GPL_2_0_with_font_exception = "GPL-2.0-with-font-exception"
    GPL_2_0 = "GPL-2.0"
    CECILL_2_1 = "CECILL-2.1"
    GPL_3_0 = "GPL-3.0"
    GPL_3_0_ = "GPL-3.0+"
    GPL_3_0_with_autoconf_exception = "GPL-3.0-with-autoconf-exception"
    GPL_3_0_only = "GPL-3.0-only"
    Hippocratic_2_1 = "Hippocratic-2.1"
    HPND = "HPND"
    HTMLTIDY = "HTMLTIDY"
    GPL_3_0_with_GCC_exception = "GPL-3.0-with-GCC-exception"
    HaskellReport = "HaskellReport"
    GPL_3_0_or_later = "GPL-3.0-or-later"
    ICU = "ICU"
    ImageMagick = "ImageMagick"
    iMatix = "iMatix"
    IBM_pibs = "IBM-pibs"
    Intel_ACPI = "Intel-ACPI"
    Intel = "Intel"
    Info_ZIP = "Info-ZIP"
    IPA = "IPA"
    IJG = "IJG"
    ISC = "ISC"
    JasPer_2_0 = "JasPer-2.0"
    JPNIC = "JPNIC"
    JSON = "JSON"
    LAL_1_2 = "LAL-1.2"
    LAL_1_3 = "LAL-1.3"
    Latex2e = "Latex2e"
    Leptonica = "Leptonica"
    HPND_sell_variant = "HPND-sell-variant"
    LGPL_2_0_only = "LGPL-2.0-only"
    LGPL_2_0_or_later = "LGPL-2.0-or-later"
    Imlib2 = "Imlib2"
    IPL_1_0 = "IPL-1.0"
    LGPL_2_1_only = "LGPL-2.1-only"
    LGPL_2_1_or_later = "LGPL-2.1-or-later"
    LGPL_2_0_ = "LGPL-2.0+"
    LGPL_2_0 = "LGPL-2.0"
    CECILL_B = "CECILL-B"
    LGPL_3_0_or_later = "LGPL-3.0-or-later"
    LGPL_3_0_only = "LGPL-3.0-only"
    LGPLLR = "LGPLLR"
    libpng_2_0 = "libpng-2.0"
    Libpng = "Libpng"
    libselinux_1_0 = "libselinux-1.0"
    LGPL_3_0_ = "LGPL-3.0+"
    EFL_1_0 = "EFL-1.0"
    libtiff = "libtiff"
    GFDL_1_3_no_invariants_or_later = "GFDL-1.3-no-invariants-or-later"
    LiLiQ_Rplus_1_1 = "LiLiQ-Rplus-1.1"
    LiLiQ_R_1_1 = "LiLiQ-R-1.1"
    LPL_1_0 = "LPL-1.0"
    LiLiQ_P_1_1 = "LiLiQ-P-1.1"
    Linux_OpenIB = "Linux-OpenIB"
    LPPL_1_0 = "LPPL-1.0"
    LPPL_1_2 = "LPPL-1.2"
    LPPL_1_3a = "LPPL-1.3a"
    LPL_1_02 = "LPL-1.02"
    LPPL_1_3c = "LPPL-1.3c"
    MakeIndex = "MakeIndex"
    LGPL_2_1_ = "LGPL-2.1+"
    LPPL_1_1 = "LPPL-1.1"
    MIT_CMU = "MIT-CMU"
    MirOS = "MirOS"
    MIT_advertising = "MIT-advertising"
    MIT_Modern_Variant = "MIT-Modern-Variant"
    MIT = "MIT"
    MIT_enna = "MIT-enna"
    MIT_open_group = "MIT-open-group"
    MIT_feh = "MIT-feh"
    MITNFA = "MITNFA"
    MPL_1_0 = "MPL-1.0"
    mpich2 = "mpich2"
    MPL_2_0 = "MPL-2.0"
    MPL_2_0_no_copyleft_exception = "MPL-2.0-no-copyleft-exception"
    MS_RL = "MS-RL"
    MTLL = "MTLL"
    MPL_1_1 = "MPL-1.1"
    MulanPSL_2_0 = "MulanPSL-2.0"
    Motosoto = "Motosoto"
    Mup = "Mup"
    MulanPSL_1_0 = "MulanPSL-1.0"
    NAIST_2003 = "NAIST-2003"
    Naumen = "Naumen"
    Multics = "Multics"
    NBPL_1_0 = "NBPL-1.0"
    NCSA = "NCSA"
    Net_SNMP = "Net-SNMP"
    NetCDF = "NetCDF"
    NASA_1_3 = "NASA-1.3"
    NGPL = "NGPL"
    NIST_PD_fallback = "NIST-PD-fallback"
    NIST_PD = "NIST-PD"
    Newsletr = "Newsletr"
    NLPL = "NLPL"
    Nokia = "Nokia"
    NOSL = "NOSL"
    Noweb = "Noweb"
    NLOD_1_0 = "NLOD-1.0"
    NPL_1_0 = "NPL-1.0"
    NCGL_UK_2_0 = "NCGL-UK-2.0"
    NRL = "NRL"
    NTP_0 = "NTP-0"
    NTP = "NTP"
    GFDL_1_3_or_later = "GFDL-1.3-or-later"
    Nunit = "Nunit"
    O_UDA_1_0 = "O-UDA-1.0"
    NPL_1_1 = "NPL-1.1"
    OCCT_PL = "OCCT-PL"
    ODC_By_1_0 = "ODC-By-1.0"
    OFL_1_0_no_RFN = "OFL-1.0-no-RFN"
    OCLC_2_0 = "OCLC-2.0"
    OFL_1_0_RFN = "OFL-1.0-RFN"
    OFL_1_0 = "OFL-1.0"
    OFL_1_1_no_RFN = "OFL-1.1-no-RFN"
    OFL_1_1_RFN = "OFL-1.1-RFN"
    OFL_1_1 = "OFL-1.1"
    OGDL_Taiwan_1_0 = "OGDL-Taiwan-1.0"
    OGC_1_0 = "OGC-1.0"
    OGL_UK_1_0 = "OGL-UK-1.0"
    OGL_UK_2_0 = "OGL-UK-2.0"
    OGL_UK_3_0 = "OGL-UK-3.0"
    OGTSL = "OGTSL"
    OLDAP_1_1 = "OLDAP-1.1"
    OLDAP_1_2 = "OLDAP-1.2"
    OLDAP_1_3 = "OLDAP-1.3"
    OGL_Canada_2_0 = "OGL-Canada-2.0"
    OLDAP_2_0_1 = "OLDAP-2.0.1"
    OLDAP_2_0 = "OLDAP-2.0"
    OLDAP_2_1 = "OLDAP-2.1"
    OLDAP_2_2_1 = "OLDAP-2.2.1"
    OLDAP_2_2_2 = "OLDAP-2.2.2"
    OLDAP_2_2 = "OLDAP-2.2"
    ODbL_1_0 = "ODbL-1.0"
    OLDAP_2_4 = "OLDAP-2.4"
    OLDAP_1_4 = "OLDAP-1.4"
    OLDAP_2_3 = "OLDAP-2.3"
    OLDAP_2_7 = "OLDAP-2.7"
    OLDAP_2_8 = "OLDAP-2.8"
    OML = "OML"
    OpenSSL = "OpenSSL"
    OLDAP_2_6 = "OLDAP-2.6"
    OPL_1_0 = "OPL-1.0"
    OSL_1_0 = "OSL-1.0"
    OSL_1_1 = "OSL-1.1"
    OSL_2_0 = "OSL-2.0"
    OSET_PL_2_1 = "OSET-PL-2.1"
    OSL_2_1 = "OSL-2.1"
    Parity_6_0_0 = "Parity-6.0.0"
    Parity_7_0_0 = "Parity-7.0.0"
    PDDL_1_0 = "PDDL-1.0"
    PHP_3_0 = "PHP-3.0"
    OSL_3_0 = "OSL-3.0"
    Plexus = "Plexus"
    MS_PL = "MS-PL"
    PolyForm_Small_Business_1_0_0 = "PolyForm-Small-Business-1.0.0"
    PolyForm_Noncommercial_1_0_0 = "PolyForm-Noncommercial-1.0.0"
    PSF_2_0 = "PSF-2.0"
    psfrag = "psfrag"
    PostgreSQL = "PostgreSQL"
    psutils = "psutils"
    Qhull = "Qhull"
    QPL_1_0 = "QPL-1.0"
    Rdisc = "Rdisc"
    Python_2_0 = "Python-2.0"
    RPL_1_1 = "RPL-1.1"
    RPL_1_5 = "RPL-1.5"
    RHeCos_1_1 = "RHeCos-1.1"
    RSA_MD = "RSA-MD"
    RSCPL = "RSCPL"
    Ruby = "Ruby"
    SAX_PD = "SAX-PD"
    Saxpath = "Saxpath"
    SCEA = "SCEA"
    Sendmail_8_23 = "Sendmail-8.23"
    Sendmail = "Sendmail"
    SGI_B_1_0 = "SGI-B-1.0"
    SGI_B_1_1 = "SGI-B-1.1"
    SGI_B_2_0 = "SGI-B-2.0"
    SHL_0_5 = "SHL-0.5"
    SHL_0_51 = "SHL-0.51"
    SimPL_2_0 = "SimPL-2.0"
    SISSL_1_2 = "SISSL-1.2"
    SISSL = "SISSL"
    Sleepycat = "Sleepycat"
    SMLNJ = "SMLNJ"
    SMPPL = "SMPPL"
    SNIA = "SNIA"
    Spencer_86 = "Spencer-86"
    Spencer_94 = "Spencer-94"
    Spencer_99 = "Spencer-99"
    SPL_1_0 = "SPL-1.0"
    SSH_OpenSSH = "SSH-OpenSSH"
    PHP_3_01 = "PHP-3.01"
    SSH_short = "SSH-short"
    MIT_0 = "MIT-0"
    RPSL_1_0 = "RPSL-1.0"
    SWL = "SWL"
    SugarCRM_1_1_3 = "SugarCRM-1.1.3"
    TCL = "TCL"
    TCP_wrappers = "TCP-wrappers"
    SSPL_1_0 = "SSPL-1.0"
    TMate = "TMate"
    TOSL = "TOSL"
    TORQUE_1_1 = "TORQUE-1.1"
    TAPR_OHL_1_0 = "TAPR-OHL-1.0"
    UCL_1_0 = "UCL-1.0"
    Unicode_DFS_2015 = "Unicode-DFS-2015"
    Unicode_DFS_2016 = "Unicode-DFS-2016"
    Unicode_TOU = "Unicode-TOU"
    TU_Berlin_1_0 = "TU-Berlin-1.0"
    UPL_1_0 = "UPL-1.0"
    Unlicense = "Unlicense"
    VOSTROM = "VOSTROM"
    Vim = "Vim"
    VSL_1_0 = "VSL-1.0"
    W3C_20150513 = "W3C-20150513"
    W3C = "W3C"
    W3C_19980720 = "W3C-19980720"
    Wsuipa = "Wsuipa"
    Watcom_1_0 = "Watcom-1.0"
    WTFPL = "WTFPL"
    X11 = "X11"
    Xerox = "Xerox"
    XFree86_1_1 = "XFree86-1.1"
    xinetd = "xinetd"
    Xnet = "Xnet"
    xpp = "xpp"
    XSkat = "XSkat"
    YPL_1_0 = "YPL-1.0"
    YPL_1_1 = "YPL-1.1"
    Zed = "Zed"
    Zend_2_0 = "Zend-2.0"
    TU_Berlin_2_0 = "TU-Berlin-2.0"
    Zimbra_1_4 = "Zimbra-1.4"
    zlib_acknowledgement = "zlib-acknowledgement"
    Zlib = "Zlib"
    ZPL_1_1 = "ZPL-1.1"
    ZPL_2_0 = "ZPL-2.0"
    ZPL_2_1 = "ZPL-2.1"
    wxWindows = "wxWindows"
    Zimbra_1_3 = "Zimbra-1.3"
    gSOAP_1_3b = "gSOAP-1.3b"
    Interbase_1_0 = "Interbase-1.0"
    LGPL_2_1 = "LGPL-2.1"
    LGPL_3_0 = "LGPL-3.0"
    NPOSL_3_0 = "NPOSL-3.0"
    OLDAP_2_5 = "OLDAP-2.5"
    StandardML_NJ = "StandardML-NJ"
    field_389_exception = "389-exception"
    Autoconf_exception_2_0 = "Autoconf-exception-2.0"
    Autoconf_exception_3_0 = "Autoconf-exception-3.0"
    Bison_exception_2_2 = "Bison-exception-2.2"
    Bootloader_exception = "Bootloader-exception"
    Classpath_exception_2_0 = "Classpath-exception-2.0"
    CLISP_exception_2_0 = "CLISP-exception-2.0"
    DigiRule_FOSS_exception = "DigiRule-FOSS-exception"
    eCos_exception_2_0 = "eCos-exception-2.0"
    Fawkes_Runtime_exception = "Fawkes-Runtime-exception"
    FLTK_exception = "FLTK-exception"
    Font_exception_2_0 = "Font-exception-2.0"
    freertos_exception_2_0 = "freertos-exception-2.0"
    GCC_exception_2_0 = "GCC-exception-2.0"
    GCC_exception_3_1 = "GCC-exception-3.1"
    gnu_javamail_exception = "gnu-javamail-exception"
    GPL_3_0_linking_exception = "GPL-3.0-linking-exception"
    GPL_3_0_linking_source_exception = "GPL-3.0-linking-source-exception"
    GPL_CC_1_0 = "GPL-CC-1.0"
    i2p_gpl_java_exception = "i2p-gpl-java-exception"
    LGPL_3_0_linking_exception = "LGPL-3.0-linking-exception"
    Libtool_exception = "Libtool-exception"
    Linux_syscall_note = "Linux-syscall-note"
    LLVM_exception = "LLVM-exception"
    LZMA_exception = "LZMA-exception"
    mif_exception = "mif-exception"
    Nokia_Qt_exception_1_1 = "Nokia-Qt-exception-1.1"
    OCaml_LGPL_linking_exception = "OCaml-LGPL-linking-exception"
    OCCT_exception_1_0 = "OCCT-exception-1.0"
    OpenJDK_assembly_exception_1_0 = "OpenJDK-assembly-exception-1.0"
    openvpn_openssl_exception = "openvpn-openssl-exception"
    PS_or_PDF_font_exception_20170817 = "PS-or-PDF-font-exception-20170817"
    Qt_GPL_exception_1_0 = "Qt-GPL-exception-1.0"
    Qt_LGPL_exception_1_1 = "Qt-LGPL-exception-1.1"
    Qwt_exception_1_0 = "Qwt-exception-1.0"
    SHL_2_0 = "SHL-2.0"
    SHL_2_1 = "SHL-2.1"
    Swift_exception = "Swift-exception"
    u_boot_exception_2_0 = "u-boot-exception-2.0"
    Universal_FOSS_exception_1_0 = "Universal-FOSS-exception-1.0"
    WxWindows_exception_3_1 = "WxWindows-exception-3.1"
